from tortoise.queryset import QuerySet
from tortoise.transactions import in_transaction

from quark import Message, Request
from quark.template.action import Action


class Status(Action):
    def __init__(self):
        self.reload = "table"
        self.checked_children = "正常"
        self.unchecked_children = "禁用"
        self.field_name = "status"
        self.field_value = "1"
        self.set_only_on_index_table_row(True)

    def get_api_params(self):
        return ["id"]

    async def handle(self, request: Request, query: QuerySet):
        # 获取参数
        status = request.query_params.get("status")
        if status is None:
            return Message.error("参数错误")

        id_ = request.query_params.get("id")
        if id_ is None:
            return Message.error("缺少id参数")

        # 解析状态
        if status in ("0", "false"):
            field_status = 0
        else:
            field_status = 1

        try:
            async with in_transaction() as conn:
                obj = await query.filter(id=int(id_)).using_db(conn).first()
                if not obj:
                    return Message.error("记录不存在")

                obj.status = field_status
                await obj.save(using_db=conn)
        except Exception as e:
            return Message.error(str(e))

        return Message.success("操作成功")



from typing import Any, Dict

from quark import Request

from ..utils import replace_last
from .resolves_actions import ResolvesActions
from .resolves_fields import ResolvesFields


class ResourceEdit:

    async def update_api(self, request: Request) -> str:
        """
        获取更新表单的接口地址
        """
        form_api = await self.form_api(request)
        if form_api:
            return form_api

        uri = request.url.path.split("/")
        if uri[-1] == "index":
            return replace_last(request.url.path, "/index", "/save")

        return replace_last(request.url.path, "/edit", "/save")

    async def edit_value_api(self, request: Request) -> str:
        """
        编辑页面获取数据接口地址
        """
        uri = request.url.path.split("/")
        if uri[-1] == "index":
            return replace_last(request.url.path, "/index", "/edit/values?id=${id}")

        return replace_last(request.url.path, "/edit", "/edit/values?id=${id}")

    async def update_component_render(
        self, request: Request, data: Dict[str, Any]
    ) -> Any:
        """
        渲染编辑页组件
        """
        title = await self.form_title(request)
        actions = await self.actions(request)
        form_extra_actions = await ResolvesActions(
            request=request, actions=actions
        ).form_extra_actions()
        api = await self.update_api(request)
        fields = ResolvesFields(
            request=request,
            fields=await self.fields(request),
        ).update_fields_within_components()
        form_actions = await ResolvesActions(
            request=request, actions=actions
        ).form_actions()

        return await self.form_component_render(
            request, title, form_extra_actions, api, fields, form_actions, data
        )

    async def before_editing(
        self, request: Request, data: Dict[str, Any]
    ) -> Dict[str, Any]:
        """
        编辑页面显示前回调
        """
        return data



from typing import Any
from quark import Request
from tortoise.models import Model
from quark.template.search import Select


class Status(Select):
    """状态组件"""

    def __init__(self):
        super().__init__("status", "状态")

    def apply(self, request: Request, query: Model, value: Any) -> Model:
        return query.filter(**{f"{self.column}": value})

    def options(self, request: Request):
        return [
            self.option("正常", 1),
            self.option("禁用", 0),
        ]



from typing import Optional

from pydantic import BaseModel

from ...component.statistic.statistic import Statistic


class Value(BaseModel):
    title: Optional[str] = None
    col: Optional[int] = None
    precision: Optional[int] = None

    # 记录条数
    def count(self, value: int) -> Statistic:
        return self.result(value)

    # 包含组件的结果
    def result(self, value: int) -> Statistic:
        return Statistic().set_title(self.title).set_value(value)



from typing import Any, Dict, List

from quark import Request

from ..component.card.card import Card
from ..component.tabs.tabs import Tabs
from ..utils import replace_last
from .resolves_actions import ResolvesActions
from .resolves_fields import ResolvesFields


class ResourceDetail:

    async def detail_value_api(self, request: Request) -> str:
        """
        获取详情页数据接口地址
        """
        uri = request.url.path.split("/")
        if uri[-1] == "index":
            return replace_last(request.url.path, "/index", "/detail/values?id=${id}")

        return replace_last(request.url.path, "/detail", "/detail/values?id=${id}")

    async def detail_title(self, request: Request) -> str:
        """
        获取详情页标题
        """
        return f"{self.title}详情"

    async def detail_component_render(
        self, request: Request, data: Dict[str, Any]
    ) -> Card:
        """
        渲染详情页组件
        """
        title = await self.detail_title(request)
        actions = await self.actions(request)
        detail_extra_actions = await ResolvesActions(
            request=request, actions=actions
        ).detail_extra_actions()

        detail_actions = await ResolvesActions(
            request=request, actions=actions
        ).detail_actions()

        fields = ResolvesFields(
            request=request,
            fields=await self.fields(request),
        ).detail_fields_within_components(
            None,
            data,
            detail_actions,
        )

        return await self.detail_within_card(
            request, title, detail_extra_actions, fields, detail_actions, data
        )

    async def detail_within_card(
        self,
        request: Request,
        title: str,
        extra: Any,
        fields: List[Any],
        actions: List[Any],
        data: Dict[str, Any],
    ) -> Card:
        """
        在卡片中渲染详情页
        """
        return (
            Card()
            .set_title(title)
            .set_header_bordered(True)
            .set_extra(extra)
            .set_body(fields)
        )

    async def detail_within_tabs(
        self,
        request: Request,
        title: str,
        extra: Any,
        fields: List[Any],
        actions: List[Any],
        data: Dict[str, Any],
    ) -> Tabs:
        """
        在标签页中渲染详情页
        """
        return Tabs().set_tab_panes(fields).set_tab_bar_extra_content(extra)

    async def before_detail_showing(
        self, request: Request, data: Dict[str, Any]
    ) -> Dict[str, Any]:
        """
        详情页显示前回调
        """
        return data



from pydantic import Field, model_validator
from typing import Any, Dict, Optional, Union, List
from .col import Col
from ..component import Component

class Row(Component):
    component: str = "row"
    align: str = Field("", description="垂直对齐方式")
    gutter: Optional[Union[int, Dict[str, int], List[int]]] = Field(None, description="栅格间隔")
    justify: str = Field("", description="水平排列方式")
    wrap: bool = Field(True, description="是否自动换行")
    col: Optional[Col] = Field(None, description="设置列")
    body: Optional[Any] = Field(None, description="内容")
    style: Optional[Dict[str, Any]] = Field(None, description="样式")

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        return self

    def set_style(self, style: Dict[str, Any]):
        # 设置样式
        self.style = style
        return self

    def set_align(self, align: str):
        # 设置垂直对齐方式
        self.align = align
        return self

    def set_gutter(self, gutter: Union[int, Dict[str, int], List[int]]):
        # 设置栅格间隔
        self.gutter = gutter
        return self

    def set_justify(self, justify: str):
        # 设置水平排列方式
        self.justify = justify
        return self

    def set_wrap(self, wrap: bool):
        # 设置是否自动换行
        self.wrap = wrap
        return self

    def set_col(self, col: Col):
        # 设置列
        self.col = col
        return self

    def set_body(self, body: Any):
        # 设置内容
        self.body = body
        return self


from tortoise.queryset import QuerySet

from quark import Message, Request
from quark.template.action import Action


class SelectOptions(Action):
    def __init__(self):
        pass

    async def handle(self, request: Request, query: QuerySet):
        return Message.success("操作成功")
